// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function OrdinaryLeastSquares(X, y){
  var B;
  var Xt, t1, t2, t3;

  /* Formula: B = (X^t * X)^-1 * X^t * y */
  Xt = TransposeToNew(X);
  t1 = MultiplyToNew(Xt, X);
  t2 = CreateSquareMatrix(NumberOfRows(t1));
  Inverse(t1, t2);
  t3 = MultiplyToNew(t2, Xt);
  B = MultiplyToNew(t3, y);

  delete(Xt);
  delete(t1);
  delete(t2);
  delete(t3);

  return B;
}
function OrdinaryLeastSquaresWithIntercept(X, y){
  var B;
  var Ones, XWithOnes;

  Ones = CreateMatrix(NumberOfRows(X), 1);
  Fill(Ones, 1);
  XWithOnes = ColumnCombineMatricesToNew(Ones, X);

  B = OrdinaryLeastSquares(XWithOnes, y);

  delete(XWithOnes);
  delete(Ones);

  return B;
}
function test(){
  var failures;
  var y, X, B;
  var yarray, xarray;
  var i;

  failures = CreateNumberReference(0);

  /* Height (m) */
  yarray = nStringToNumberArray("52.21, 53.12, 54.48, 55.84, 57.20, 58.57, 59.93, 61.29, 63.11, 64.47, 66.28, 68.10, 69.92, 72.19, 74.46".split(''));

  y = CreateMatrix(yarray.length, 1);

  for(i = 0; i < yarray.length; i = i + 1){
    y.r[i].c[0] = yarray[i];
  }

  /* Weight (kg) */
  xarray = nStringToNumberArray("1.47, 1.50, 1.52, 1.55, 1.57, 1.60, 1.63, 1.65, 1.68, 1.70, 1.73, 1.75, 1.78, 1.80, 1.83".split(''));

  X = CreateMatrix(xarray.length, 2);

  for(i = 0; i < xarray.length; i = i + 1){
    X.r[i].c[0] = xarray[i];
    X.r[i].c[1] = Math.pow(xarray[i], 2);
  }

  B = OrdinaryLeastSquaresWithIntercept(X, y);

  AssertTrue(EpsilonCompare(B.r[0].c[0], 128.8128, 0.0001), failures);
  AssertTrue(EpsilonCompare(B.r[1].c[0],  -143.1620, 0.0001), failures);
  AssertTrue(EpsilonCompare(B.r[2].c[0], 61.9603, 0.0001), failures);

  B = OrdinaryLeastSquares(X, y);

  return failures.numberValue;
}
function Add(a, b){
  var m, n;
  var d;

  d = NumberOfRows(a);
  for(m = 0; m < d; m = m + 1){
    for(n = 0; n < d; n = n + 1){
      a.r[m].c[n] = a.r[m].c[n] + b.r[m].c[n];
    }
  }
}
function Assign(A, B){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = B.r[m].c[n];
    }
  }
}
function Subtract(a, b){
  var m, n;
  var r, c;

  r = NumberOfRows(a);
  c = NumberOfColumns(a);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      a.r[m].c[n] = a.r[m].c[n] - b.r[m].c[n];
    }
  }
}
function SubtractToNew(a, b){
  var X;

  X = CreateCopyOfMatrix(a);
  Subtract(X, b);

  return X;
}
function ScalarMultiply(A, b){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = b*A.r[m].c[n];
    }
  }
}
function ScalarDivide(A, b){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);
  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = A.r[m].c[n]/b;
    }
  }
}
function ElementWisePower(A, p){
  var m, n;
  var r, c;

  r = NumberOfRows(A);
  c = NumberOfColumns(A);

  for(m = 0; m < r; m = m + 1){
    for(n = 0; n < c; n = n + 1){
      A.r[m].c[n] = Math.pow(A.r[m].c[n], p);
    }
  }
}
function ScalarMultiplyToNew(A, b){
  var matrix;

  matrix = CreateMatrix(NumberOfRows(A), NumberOfColumns(A));
  ScalarMultiply(matrix, b);

  return matrix;
}
function MultiplyToNew(a, b){
  var rows, cols;
  var x;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(b);
  x = CreateMatrix(rows, cols);
  Multiply(x, a, b);

  return x;
}
function Multiply(x, a, b){
  var m, n;
  var rows, cols, d;
  var i, s;

  rows = NumberOfRows(a);
  cols = NumberOfColumns(b);
  d = NumberOfColumns(a);

  for(m = 0; m < rows; m = m + 1){
    for(n = 0; n < cols; n = n + 1){
      s = 0;

      for(i = 0; i < d; i = i + 1){
        s = s + a.r[m].c[i]*b.r[i].c[n];
      }

      x.r[m].c[n] = s;
    }
  }
}
function CreateSquareMatrix(d){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = d;
  for(m = 0; m < d; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = d;
    for(n = 0; n < d; n = n + 1){
      matrix.r[m].c[n] = 0;
    }
  }

  return matrix;
}
function CreateMatrix(rows, cols){
  var m, n;
  var matrix;

  matrix = {};
  matrix.r = [];
  matrix.r.length = rows;
  for(m = 0; m < rows; m = m + 1){
    matrix.r[m] = {};
    matrix.r[m].c = [];
    matrix.r[m].c.length = cols;
    for(n = 0; n < cols; n = n + 1){
      matrix.r[m].c[n] = 0;
    }
  }

  return matrix;
}
function CreateIdentityMatrix(d){
  var m;
  var matrix;

  matrix = CreateSquareMatrix(d);
  Fill(matrix, 0);

  for(m = 0; m < d; m = m + 1){
    matrix.r[m].c[m] = 1;
  }

  return matrix;
}
function Transpose(a){
  var m, n;
  var rows, cols, tmp;

  cols = NumberOfRows(a);
  rows = NumberOfColumns(a);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      tmp = a.r[n].c[m];
      a.r[n].c[m] = a.r[m].c[n];
      a.r[m].c[n] = tmp;
    }
  }
}
function TransposeToNew(a){
  var m, n;
  var rows, cols;
  var c;

  cols = NumberOfRows(a);
  rows = NumberOfColumns(a);

  c = CreateMatrix(rows, cols);

  for(m = 0; m < cols; m = m + 1){
    for(n = 0; n < rows; n = n + 1){
      c.r[n].c[m] = a.r[m].c[n];
    }
  }

  return c;
}
function CofactorOfMatrix(mat, temp, p, q, n){
  var i, j;
  var row, col;

  i = 0;
  j = 0;

  for(row = 0; row < n; row = row + 1){
    for(col = 0; col < n; col = col + 1){
      if(row != p && col != q){
        temp.r[i].c[j] = mat.r[row].c[col];
        j = j + 1;

        if(j == n - 1){
          j = 0;
          i = i + 1;
        }
      }
    }
  }
}
function DeterminantOfSubmatrix(mat, n){
  var D, f, sign;
  var temp;

  D = 0;

  if(n == 1){
    D = mat.r[0].c[0];
  }else{
    temp = CreateSquareMatrix(n);

    sign = 1;

    for(f = 0; f < n; f = f + 1){
      CofactorOfMatrix(mat, temp, 0, f, n);
      D = D + sign*mat.r[0].c[f]*DeterminantOfSubmatrix(temp, n - 1);
      sign =  -sign;
    }

    DeleteMatrix(temp);
  }

  return D;
}
function Determinant(m){
  var D, n;

  n = NumberOfRows(m);
  D = DeterminantOfSubmatrix(m, n);

  return D;
}
function Adjoint(A, adj){
  var n, sign;
  var cofactors;
  var i, j;

  n = A.r.length;

  if(n == 1){
    adj.r[0].c[0] = 1;
  }else{
    cofactors = CreateSquareMatrix(n);

    for(i = 0; i < n; i = i + 1){
      for(j = 0; j < n; j = j + 1){
        CofactorOfMatrix(A, cofactors, i, j, n);

        if((i + j)%2 == 0){
          sign = 1;
        }else{
          sign =  -1;
        }

        adj.r[j].c[i] = sign*DeterminantOfSubmatrix(cofactors, n - 1);
      }
    }

    DeleteMatrix(cofactors);
  }
}
function Inverse(A, inverseResult){
  var success;
  var adj;
  var n, i, j;
  var det;

  if(NumberOfColumns(A) == NumberOfRows(A)){
    n = NumberOfColumns(A);

    det = Determinant(A);
    if(det != 0){
      adj = CreateSquareMatrix(n);
      Adjoint(A, adj);

      for(i = 0; i < n; i = i + 1){
        for(j = 0; j < n; j = j + 1){
          inverseResult.r[i].c[j] = adj.r[i].c[j]/det;
        }
      }

      success = true;
      DeleteMatrix(adj);
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function IsSymmetric(A){
  var N;
  var i, j;
  var is, done;

  N = NumberOfRows(A);

  done = false;
  is = true;
  for(i = 0; i < N &&  !done ; i = i + 1){
    for(j = 0; j < i &&  !done ; j = j + 1){
      if(A.r[i].c[j] != A.r[j].c[i]){
        is = false;
        done = true;
      }
    }
  }

  return is;
}
function IsSquare(A){
  var is;

  if(NumberOfRows(A) == NumberOfColumns(A)){
    is = true;
  }else{
    is = false;
  }

  return is;
}
function Cholesky(A, L){
  var success;
  var N;
  var i, j, k, s;

  Clear(L);

  if(IsSquare(A) && IsSymmetric(A)){
    success = true;

    N = NumberOfRows(A);

    for(i = 0; i < N && success; i = i + 1){
      for(j = 0; j <= i && success; j = j + 1){
        s = 0;
        for(k = 0; k < j; k = k + 1){
          s = s + L.r[i].c[k]*L.r[j].c[k];
        }
        if(i == j){
          L.r[i].c[i] = Math.sqrt(A.r[i].c[i] - s);
        }else{
          L.r[i].c[j] = 1/L.r[j].c[j]*(A.r[i].c[j] - s);
        }
      }
      if(L.r[i].c[i] <= 0){
        success = false;
      }
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function Clear(a){
  Fill(a, 0);
}
function Fill(a, value){
  var m, n;

  for(m = 0; m < NumberOfRows(a); m = m + 1){
    for(n = 0; n < NumberOfColumns(a); n = n + 1){
      a.r[m].c[n] = value;
    }
  }
}
function Element(matrix, m, n){
  return matrix.r[m].c[n];
}
function Trace(a){
  var m;
  var d, tr;

  tr = 0;

  d = a.r.length;
  for(m = 0; m < d; m = m + 1){
    tr = tr + a.r[m].c[m];
  }

  return tr;
}
function ColumnCombineMatricesToNew(A, B){
  var X;
  var m, n;

  X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A) + NumberOfColumns(B));

  for(m = 0; m < NumberOfRows(A); m = m + 1){
    for(n = 0; n < NumberOfColumns(A); n = n + 1){
      X.r[m].c[n] = A.r[m].c[n];
    }
  }

  for(m = 0; m < NumberOfRows(B); m = m + 1){
    for(n = 0; n < NumberOfColumns(B); n = n + 1){
      X.r[m].c[NumberOfColumns(A) + n] = B.r[m].c[n];
    }
  }

  return X;
}
function NumberOfRows(A){
  return A.r.length;
}
function NumberOfColumns(A){
  return A.r[0].c.length;
}
function CharacteristicPolynomial(A){
  var coeffs;
  var dummy;

  dummy = CreateSquareMatrix(0);

  coeffs = CharacteristicPolynomialWithInverse(A, false, dummy);

  DeleteMatrix(dummy);

  return coeffs;
}
function CharacteristicPolynomialWithInverse(A, returnInverse, AInverse){
  var coeffs;
  var Mk, Mkm1, t1, I;
  var n, k;

  n = NumberOfRows(A);
  coeffs = [];
  coeffs.length = n + 1;
  coeffs[n] = 1;
  Mkm1 = CreateSquareMatrix(n);
  Fill(Mkm1, 0);
  I = CreateIdentityMatrix(n);
  Mk = CreateSquareMatrix(n);
  t1 = CreateSquareMatrix(n);

  for(k = 1; k <= n; k = k + 1){
    /* M_k = A * M_(k-1) + c_(n-k+1) * I */
    Multiply(Mk, A, Mkm1);
    Assign(t1, I);
    ScalarMultiply(t1, coeffs[n - k + 1]);
    Add(Mk, t1);

    /* c_(n-k) = -1/k * trace(A * M_k) */
    Multiply(t1, A, Mk);
    coeffs[n - k] =  -1/k*Trace(t1);

    /* done */
    Assign(Mkm1, Mk);

    if(k == n && returnInverse){
      Assign(AInverse, Mk);
      ScalarDivide(AInverse,  -coeffs[0]);
    }
  }

  DeleteMatrix(Mkm1);
  DeleteMatrix(I);
  DeleteMatrix(Mk);
  DeleteMatrix(t1);

  return coeffs;
}
function InverseAlt2(A){
  var inverse;
  var coeffs;

  inverse = CreateSquareMatrix(NumberOfRows(A));
  coeffs = CharacteristicPolynomialWithInverse(A, true, inverse);
  delete(coeffs);

  return inverse;
}
function Eigenvalues(A, eigenValuesReference){
  var p;
  var success;

  p = CharacteristicPolynomial(A);
  success = FindRoots(p, eigenValuesReference);

  return success;
}
function GaussianElimination(A){
  var h, k, m, n, maxElement, i, j, max, maxCandidate, f;

  m = NumberOfRows(A);
  n = NumberOfColumns(A);

  h = 0;
  k = 0;
  for(; h < m && k < n; ){
    maxElement = h;
    max = 0;
    for(i = h; i < m; i = i + 1){
      maxCandidate = Math.abs(Element(A, i, k));
      if(max < maxCandidate){
        maxElement = i;
        max = maxCandidate;
      }
    }
    if(A.r[maxElement].c[k] == 0){
      k = k + 1;
    }else{
      SwapRows(A, h, maxElement);
      for(i = h + 1; i < m; i = i + 1){
        f = Element(A, i, k)/Element(A, h, k);
        A.r[i].c[k] = 0;
        for(j = k + 1; j < n; j = j + 1){
          A.r[i].c[j] = Element(A, i, j) - Element(A, h, j)*f;
        }
      }
      h = h + 1;
      k = k + 1;
    }
  }
}
function GaussianEliminationToNew(A){
  var X;

  X = CreateCopyOfMatrix(A);
  GaussianElimination(X);

  return X;
}
function CreateCopyOfMatrix(A){
  var X;

  X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A));
  Assign(X, A);

  return X;
}
function SwapRows(A, to, from){
  var n;
  var c, t;

  c = NumberOfRows(A);
  for(n = 0; n < c; n = n + 1){
    t = A.r[to].c[n];
    A.r[to].c[n] = A.r[from].c[n];
    A.r[from].c[n] = t;
  }
}
function Eigenvectors(A, eigenVectorsReference){
  var Id, t1, B, v;
  var eigenVectorsResult;
  var success;
  var eigenValuesReference;
  var i, lambda, j, N, x, k;
  var ev;

  N = NumberOfRows(A);

  eigenValuesReference = {};
  success = Eigenvalues(A, eigenValuesReference);

  eigenVectorsResult = [];
  eigenVectorsResult.length = N;

  if(success){
    ev = eigenValuesReference.numberArray;

    Id = CreateIdentityMatrix(N);
    t1 = CreateSquareMatrix(N);
    B = CreateSquareMatrix(N);

    for(j = 0; j < ev.length && success; j = j + 1){
      lambda = ev[j];

      /* B = A - lambda * Id */
      Assign(t1, Id);
      ScalarMultiply(t1, lambda);

      Assign(B, A);
      Subtract(B, t1);

      GaussianElimination(B);

      v = CreateMatrix(N, 1);
      v.r[N - 1].c[0] = 1;
      for(i = N - 2; i >= 0 && success; i = i - 1){
        if( !RowIsZero(B, i) ){
          x = 0;

          for(k = N - 1; k > i; k = k - 1){
            x = x - Element(B, i, k)*Element(v, k, 0);
          }

          v.r[i].c[0] = x/Element(B, i, i);
        }else{
          success = false;
        }
      }

      eigenVectorsResult[j] = v;
    }

    DeleteMatrix(t1);
    DeleteMatrix(B);
    DeleteMatrix(Id);

    eigenVectorsReference.matrices = eigenVectorsResult;
  }

  return success;
}
function RowIsZero(X, r){
  var isZero;
  var columns, i;

  isZero = true;

  columns = NumberOfColumns(X);
  for(i = 0; i < columns && isZero; i = i + 1){
    if(Element(X, r, i) != 0){
      isZero = false;
    }
  }

  return isZero;
}
function DeleteMatrix(X){
  var m, rows;

  rows = NumberOfRows(X);
  for(m = 0; m < rows; m = m + 1){
    delete(X.r[m]);
  }

  delete(X.r);
  delete(X);
}
function CreateDiagonalMatrixFromArray(array){
  var m;
  var matrix;

  matrix = CreateSquareMatrix(array.length);
  Fill(matrix, 0);

  for(m = 0; m < array.length; m = m + 1){
    matrix.r[m].c[m] = array[m];
  }

  return matrix;
}
function CreateMatrixFromRowCopies(row, times){
  var m, n;
  var matrix;

  matrix = CreateMatrix(times, row.length);

  for(m = 0; m < times; m = m + 1){
    for(n = 0; n < row.length; n = n + 1){
      matrix.r[m].c[n] = row[n];
    }
  }

  return matrix;
}
function ExtractDiagonal(X){
  var diag;
  var n, i;

  n = NumberOfRows(X);
  diag = [];
  diag.length = n;

  for(i = 0; i < n; i = i + 1){
    diag[i] = X.r[i].c[i];
  }

  return diag;
}
function MatrixEqualsEpsilon(b, f, epsilon){
  var x, y, columns, rows;
  var equals;

  equals = true;

  if(NumberOfRows(b) == NumberOfColumns(f) && NumberOfColumns(b) == NumberOfColumns(f)){
    columns = NumberOfColumns(b);
    rows = NumberOfRows(b);

    for(x = 0; x < columns; x = x + 1){
      for(y = 0; y < rows; y = y + 1){
        equals = equals && EpsilonCompare(b.r[x].c[y], f.r[x].c[y], epsilon);
      }
    }
  }else{
    equals = false;
  }

  return equals;
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !StringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;
  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function FindRoots(p, rootsReference){
  return LaguerresMethodWithRepeatedDivision(p, 100, 0.000001, 0, rootsReference);
}
function LaguerresMethodWithRepeatedDivision(p, maxIterations, precision, guess, rootsReference){
  var n, nr, xk;
  var x;
  var q, r, d;
  var success;
  var xkReference;
  n = pDegree(p);

  x = [];
  x.length = n;

  q = pCreatePolynomial(n);
  r = pCreatePolynomial(n);
  d = pCreatePolynomial(n);

  success = true;
  xkReference = CreateNumberReference(0);

  for(nr = 0; nr < n && success; nr = nr + 1){
    success = LaguerresMethod(p, guess, maxIterations, precision, xkReference);
    if(success){
      xk = xkReference.numberValue;
      x[nr] = xk;
      pFill(d, 0);
      d[0] =  -xk;
      d[1] = 1;
      pDivide(q, r, p, d);
      pAssign(p, q);
    }
  }

  delete(q);
  delete(r);
  delete(d);
        
  rootsReference.numberArray = x;

  return success;
}
function LaguerresMethod(p, guess, maxIterations, precision, rootReference){
  var k, a, G, H, xk, denom1, denom2, denom, t1, n;
  var success;
  n = pDegree(p);
  success = true;

  xk = guess;

  for(k = 0; (k < maxIterations) && (Math.abs(pEvaluate(p, xk)) >= precision) && success; k = k + 1){
    G = pEvaluateDerivative(p, xk, 1)/pEvaluate(p, xk);
    H = Math.pow(G, 2) - pEvaluateDerivative(p, xk, 2)/pEvaluate(p, xk);
    t1 = (n - 1)*(n*H - Math.pow(G, 2));
    if(t1 >= 0){
      denom = Math.sqrt(t1);
      denom1 = G + denom;
      denom2 = G - denom;
      if(Math.abs(denom1) >= Math.abs(denom2)){
        denom = denom1;
      }else{
        denom = denom2;
      }
      a = n/denom;
      xk = xk - a;
    }else{
      success = false;
    }
  }

  if(k == maxIterations){
    success = false;
  }

  if(Math.abs(pEvaluate(p, xk)) >= precision){
    success = false;
  }

  rootReference.numberValue = xk;

  return success;
}
function CreateBooleanReference(value){
  var ref;
  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;
  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;
  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;
  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;
  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;
  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;
  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;
  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;
  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;
  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;
  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;
  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  return Factorial(n)/(Factorial(n - k)*Factorial(k));
}
function EpsilonCompareApproximateDigits(a, b, digits){
  var ad, bd, d, epsilon;
  var ret;
  if(a < 0 && b < 0 || a > 0 && b > 0){
    if(a < 0 && b < 0){
      a =  -a;
      b =  -b;
    }
    ad = Math.log10(a);
    bd = Math.log10(b);
    d = Math.max(ad, bd);
    epsilon = Math.pow(10, d - digits);
    ret = Math.abs(a - b) > epsilon;
  }else{
    ret = false;
  }

  return ret;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;
  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;
  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;
  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;
  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function StringToNumberArray(string){
  var i;
  var array;
  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function NumberArrayToString(array){
  var i;
  var string;
  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function NumberArraysEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function BooleanArraysEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function StringsEqual(a, b){
  var equal;
  var i;
  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function FillNumberArray(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillString(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillBooleanArray(a, value){
  var i;
  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillNumberArrayInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = from; i < to; i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillBooleanArrayInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = Math.max(from, 0); i < Math.min(to, a.length); i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillStringInterval(a, value, from, to){
  var i;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length){
    for(i = Math.max(from, 0); i < Math.min(to, a.length); i = i + 1){
      a[i] = value;
    }
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyNumberArray(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyBooleanArray(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyString(a){
  var i;
  var n;
  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;
  if(from >= 0 && from < a.length && to >= 0 && to < a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;
    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }
    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function IsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function CreateNumberArray(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillNumberArray(array, value);

  return array;
}
function CreateBooleanArray(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillBooleanArray(array, value);

  return array;
}
function CreateString(lengthx, value){
  var array;
  array = [];
  array.length = lengthx;
  FillString(array, value);

  return array;
}
function pAdd(a, b){
  var i, nr;
  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = a[i] + b[i];
  }
}
function pSubtract(a, b){
  var i, nr;
  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = a[i] - b[i];
  }
}
function pMultiply(c, a, b){
  var k, n, m, i;
  var av, bv;
  n = pDegree(a);
  m = pDegree(b);

  pFill(c, 0);

  for(i = 0; i <= n + m; i = i + 1){
    c[i] = 0;
    for(k = 0; k <= i && k < a.length && i - k < b.length; k = k + 1){
      av = a[k];
      bv = b[i - k];
      c[i] = c[i] + av*bv;
    }
  }
}
function pDivide(q, r, n, d){
  var t, t1;
  var tcoff, tdegree, i;
  var deg;
  var rd, dd;
  pFill(q, 0);
  pAssign(r, n);
  deg = pDegree(n);
  t = pCreatePolynomial(deg);
  t1 = pCreatePolynomial(deg);

  rd = pDegree(r);
  dd = pDegree(d);
  for(i = 0; i < deg + 1 &&  !pIsZero(r)  && rd - i >= dd; i = i + 1){
    pFill(t, 0);
    tdegree = rd - i - dd;
    tcoff = r[rd - i]/d[dd];
    t[tdegree] = tcoff;
    pAdd(q, t);
    pFill(t1, 0);
    pMultiply(t1, t, d);
    pSubtract(r, t1);
  }

  delete(t);
  delete(t1);
}
function pIsZero(a){
  var i;
  var itIsZero;
  itIsZero = true;

  for(i = 0; i < a.length; i = i + 1){
    if(a[i] != 0){
      itIsZero = false;
    }
  }

  return itIsZero;
}
function pAssign(a, b){
  var i, nr;
  nr = b.length;

  for(i = 0; i < nr; i = i + 1){
    a[i] = b[i];
  }
}
function pCreatePolynomial(deg){
  var p;
  p = [];
  p.length = deg + 1;

  pFill(p, 0);

  return p;
}
function pFill(p, value){
  var i;
  for(i = 0; i < p.length; i = i + 1){
    p[i] = value;
  }
}
function pDegree(A){
  var i;
  var deg;
  var done;
  done = false;
  deg = 0;
  for(i = A.length - 1; i >= 0 &&  !done ; i = i - 1){
    if(A[i] != 0){
      deg = i;
      done = true;
    }
  }

  return deg;
}
function pLead(A){
  var deg;
  deg = pDegree(A);

  return A[deg];
}
function pEvaluate(A, x){
  var r, i;
  r = 0;

  for(i = 0; i < A.length; i = i + 1){
    r = r + A[i]*Math.pow(x, i);
  }

  return r;
}
function pEvaluateDerivative(A, x, n){
  var r, i, v;
  r = 0;

  for(i = 0; i < A.length; i = i + 1){
    if(i - n >= 0){
      v = A[i]*Factorial(n)*Combinations(i, n)*Math.pow(x, i - n);
      r = r + v;
    }
  }

  return r;
}
function nCreateStringDecimalFromNumber(decimal){
  var stringReference;
  stringReference = {};

  /* This will succeed because base = 10. */
  nCreateStringFromNumberWithCheck(decimal, 10, stringReference);

  return stringReference.string;
}
function nCreateStringFromNumberWithCheck(decimal, base, stringReference){
  var string;
  var maximumDigits;
  var digitPosition;
  var hasPrintedPoint;
  var i, d;
  var success;
  var characterReference;
  var c;
  if(decimal == 0){
    stringReference.string = "0".split('');
    success = true;
  }else{
    characterReference = {};
    if(IsInteger(base)){
      success = true;
      string = [];
      string.length = 0;
      maximumDigits = nGetMaximumDigitsForBase(base);
      digitPosition = nGetFirstDigitPosition(decimal, base);
      decimal = Math.round(decimal*Math.pow(base, maximumDigits - digitPosition - 1));
      hasPrintedPoint = false;
      if(digitPosition < 0){
        string = AppendCharacter(string, '0');
        string = AppendCharacter(string, '.');
        hasPrintedPoint = true;
        for(i = 0; i <  -digitPosition - 1; i = i + 1){
          string = AppendCharacter(string, '0');
        }
      }
      for(i = 0; i < maximumDigits && success; i = i + 1){
        d = Math.floor(decimal/Math.pow(base, maximumDigits - i - 1));
        if( !hasPrintedPoint  && digitPosition - i + 1 == 0){
          if(decimal != 0){
            string = AppendCharacter(string, '.');
          }
          hasPrintedPoint = true;
        }
        if(decimal == 0 && hasPrintedPoint){
        }else{
          success = nGetSingleDigitCharacterFromNumberWithCheck(d, base, characterReference);
          if(success){
            c = characterReference.characterValue;
            string = AppendCharacter(string, c);
          }
        }
        if(success){
          decimal = decimal - d*Math.pow(base, maximumDigits - i - 1);
        }
      }
      if(success){
        for(i = 0; i < digitPosition - maximumDigits + 1; i = i + 1){
          string = AppendCharacter(string, '0');
        }
        stringReference.string = string;
      }
    }else{
      success = false;
    }
  }

  /* Done */
  return success;
}
function nGetMaximumDigitsForBase(base){
  var t;
  t = Math.pow(10, 15);
  return Math.floor(Math.log10(t)/Math.log10(base));
}
function nGetFirstDigitPosition(decimal, base){
  var power;
  var t;
  power = Math.ceil(Math.log10(decimal)/Math.log10(base));

  t = decimal*Math.pow(base,  -power);
  if(t < base && t >= 1){
  }else if(t >= base){
    power = power + 1;
  }else if(t < 1){
    power = power - 1;
  }

  return power;
}
function nGetSingleDigitCharacterFromNumberWithCheck(c, base, characterReference){
  var numberTable;
  var success;
  numberTable = nGetDigitCharacterTable();

  if(c < base || c < numberTable.length){
    success = true;
    characterReference.characterValue = numberTable[c];
  }else{
    success = false;
  }

  return success;
}
function nGetDigitCharacterTable(){
  var numberTable;
  numberTable = [];
  numberTable.length = 36;

  numberTable[0] = '0';
  numberTable[1] = '1';
  numberTable[2] = '2';
  numberTable[3] = '3';
  numberTable[4] = '4';
  numberTable[5] = '5';
  numberTable[6] = '6';
  numberTable[7] = '7';
  numberTable[8] = '8';
  numberTable[9] = '9';
  numberTable[10] = 'A';
  numberTable[11] = 'B';
  numberTable[12] = 'C';
  numberTable[13] = 'D';
  numberTable[14] = 'E';
  numberTable[15] = 'F';
  numberTable[16] = 'G';
  numberTable[17] = 'H';
  numberTable[18] = 'I';
  numberTable[19] = 'J';
  numberTable[20] = 'K';
  numberTable[21] = 'L';
  numberTable[22] = 'M';
  numberTable[23] = 'N';
  numberTable[24] = 'O';
  numberTable[25] = 'P';
  numberTable[26] = 'Q';
  numberTable[27] = 'R';
  numberTable[28] = 'S';
  numberTable[29] = 'T';
  numberTable[30] = 'U';
  numberTable[31] = 'V';
  numberTable[32] = 'W';
  numberTable[33] = 'X';
  numberTable[34] = 'Y';
  numberTable[35] = 'Z';

  return numberTable;
}
function nCreateNumberFromDecimalStringWithCheck(string, decimalReference, errorMessage){
  return nCreateNumberFromStringWithCheck(string, 10, decimalReference, errorMessage);
}
function nCreateNumberFromDecimalString(string){
  var doubleReference;
  var stringReference;
  var number;
  doubleReference = CreateNumberReference(0);
  stringReference = CreateStringReference("".split(''));
  nCreateNumberFromStringWithCheck(string, 10, doubleReference, stringReference);
  number = doubleReference.numberValue;

  delete(doubleReference);
  delete(stringReference);

  return number;
}
function nCreateNumberFromStringWithCheck(string, base, decimalReference, errorMessage){
  var c;
  var i, j;
  var isPositive;
  var beforeDecimalPoint, afterDecimalPoint;
  var n, validCharacters, d;
  var success;
  success = true;
  i = 0;
  isPositive = true;
  beforeDecimalPoint = 0;
  afterDecimalPoint = 0;
  n = 0;
  validCharacters = 0;

  if(base >= 2 && base <= 36){
    for(j = 0; j < string.length; j = j + 1){
      c = string[j];
      if(nCharacterIsNumberCharacterInBase(c, base) || c == '.' || c == '-'){
        validCharacters = validCharacters + 1;
      }
    }
    if(validCharacters == string.length){
      if(string.length > 0){
        c = string[i];
        if(c == '-'){
          isPositive = false;
          i = i + 1;
        }
        if(i < string.length){
          c = string[i];
          if(nCharacterIsNumberCharacterInBase(c, base)){
            for(; nCharacterIsNumberCharacterInBase(c, base) && (i < string.length); ){
              beforeDecimalPoint = beforeDecimalPoint + 1;
              i = i + 1;
              if(i < string.length){
                c = string[i];
              }
            }
            if(i < string.length){
              c = string[i];
              if(c == '.'){
                i = i + 1;
                if(i < string.length){
                  c = string[i];
                  for(; nCharacterIsNumberCharacterInBase(c, base) && (i < string.length); ){
                    afterDecimalPoint = afterDecimalPoint + 1;
                    i = i + 1;
                    if(i < string.length){
                      c = string[i];
                    }
                  }
                }else{
                  success = false;
                  errorMessage.string = "Number must have digits after the decimal point.".split('');
                }
              }
            }
          }else{
            success = false;
            errorMessage.string = "Number must start with digits (for negative numbers, after the negative sign).".split('');
          }
        }
        if(success != false){
          if(isPositive){
            i = 0;
          }else{
            i = 1;
          }
          for(j = 0; j < beforeDecimalPoint; j = j + 1){
            c = string[i];
            i = i + 1;
            d = nGetNumberFromNumberCharacterForBase(c, base);
            n = n + d*Math.pow(base, beforeDecimalPoint - j - 1);
          }
          if(afterDecimalPoint > 0){
            i = i + 1;
            for(j = 0; j < afterDecimalPoint; j = j + 1){
              c = string[i];
              i = i + 1;
              d = nGetNumberFromNumberCharacterForBase(c, base);
              n = n + d*Math.pow(base, 0 - j - 1);
            }
          }
          if( !isPositive ){
            n =  -n;
          }
          decimalReference.numberValue = n;
          success = true;
        }
      }else{
        success = false;
        errorMessage.string = "String has no content.".split('');
      }
    }else{
      success = false;
      errorMessage.string = "String contains invalid character.".split('');
    }
  }else{
    success = false;
    errorMessage.string = "Base must be from 2 to 36.".split('');
  }

  return success;
}
function nGetNumberFromNumberCharacterForBase(c, base){
  var numberTable;
  var i;
  var position;
  numberTable = nGetDigitCharacterTable();
  position = 0;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      position = i;
    }
  }

  return position;
}
function nCharacterIsNumberCharacterInBase(c, base){
  var numberTable;
  var i;
  var found;
  numberTable = nGetDigitCharacterTable();
  found = false;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      found = true;
    }
  }

  return found;
}
function nStringToNumberArray(str){
  var numberArrayReference;
  var stringReference;
  var numbers;
  numberArrayReference = {};
  stringReference = {};

  nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

  numbers = numberArrayReference.numberArray;

  delete(numberArrayReference);
  delete(stringReference);

  return numbers;
}
function nStringToNumberArrayWithCheck(str, numberArrayReference, errorMessage){
  var numberStrings;
  var numbers;
  var i;
  var numberString, trimmedNumberString;
  var success;
  var numberReference;
  numberStrings = SplitByString(str, ",".split(''));

  numbers = [];
  numbers.length = numberStrings.length;
  success = true;
  numberReference = {};

  for(i = 0; i < numberStrings.length; i = i + 1){
    numberString = numberStrings[i].string;
    trimmedNumberString = Trim(numberString);
    success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
    numbers[i] = numberReference.numberValue;
    FreeStringReference(numberStrings[i]);
    delete(trimmedNumberString);
  }

  delete(numberStrings);
  delete(numberReference);

  numberArrayReference.numberArray = numbers;

  return success;
}
function SubstringWithCheck(string, from, to, stringReference){
  var success;
  if(from < string.length && to < string.length && from <= to && from >= 0 && to >= 0){
    stringReference.string = Substring(string, from, to);
    success = true;
  }else{
    success = false;
  }

  return success;
}
function Substring(string, from, to){
  var n;
  var i;
  n = [];
  n.length = Math.max(to - from, 0);

  for(i = from; i < to; i = i + 1){
    n[i - from] = string[i];
  }

  return n;
}
function AppendString(string, s){
  var newString;
  newString = ConcatenateString(string, s);

  delete(string);

  return newString;
}
function ConcatenateString(string, s){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + s.length;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  for(i = 0; i < s.length; i = i + 1){
    newString[string.length + i] = s[i];
  }

  return newString;
}
function AppendCharacter(string, c){
  var newString;
  newString = ConcatenateCharacter(string, c);

  delete(string);

  return newString;
}
function ConcatenateCharacter(string, c){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + 1;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  newString[string.length] = c;

  return newString;
}
function SplitByCharacter(toSplit, splitBy){
  var split;
  var stringToSplitBy;
  stringToSplitBy = [];
  stringToSplitBy.length = 1;
  stringToSplitBy[0] = splitBy;

  split = SplitByString(toSplit, stringToSplitBy);

  delete(stringToSplitBy);

  return split;
}
function IndexOfCharacter(string, character, indexReference){
  var i;
  var found;
  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function SubstringEqualsWithCheck(string, from, substring, equalsReference){
  var success;
  if(from < string.length){
    success = true;
    equalsReference.booleanValue = SubstringEquals(string, from, substring);
  }else{
    success = false;
  }

  return success;
}
function SubstringEquals(string, from, substring){
  var i;
  var equal;
  equal = true;
  for(i = 0; i < substring.length && equal; i = i + 1){
    if(string[from + i] != substring[i]){
      equal = false;
    }
  }

  return equal;
}
function IndexOfString(string, substring, indexReference){
  var i;
  var found;
  found = false;
  for(i = 0; i < string.length - substring.length + 1 &&  !found ; i = i + 1){
    if(SubstringEquals(string, i, substring)){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function ContainsCharacter(string, character){
  return IndexOfCharacter(string, character, {});
}
function ContainsString(string, substring){
  return IndexOfString(string, substring, {});
}
function ToUpperCase(string){
  var i;
  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToUpperCase(string[i]);
  }
}
function ToLowerCase(string){
  var i;
  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToLowerCase(string[i]);
  }
}
function EqualsIgnoreCase(a, b){
  var equal;
  var i;
  if(a.length == b.length){
    equal = true;
    for(i = 0; i < a.length && equal; i = i + 1){
      if(charToLowerCase(a[i]) != charToLowerCase(b[i])){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function ReplacesString(string, toReplace, replaceWith){
  var result;
  var i;
  var equalsReference;
  var success;
  equalsReference = {};
  result = [];
  result.length = 0;

  for(i = 0; i < string.length; ){
    success = SubstringEqualsWithCheck(string, i, toReplace, equalsReference);
    if(success){
      success = equalsReference.booleanValue;
    }
    if(success && toReplace.length > 0){
      result = ConcatenateString(result, replaceWith);
      i = i + toReplace.length;
    }else{
      result = ConcatenateCharacter(result, string[i]);
      i = i + 1;
    }
  }

  return result;
}
function ReplaceCharacter(string, toReplace, replaceWith){
  var result;
  var i;
  result = [];
  result.length = 0;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      result = ConcatenateCharacter(result, replaceWith);
    }else{
      result = ConcatenateCharacter(result, string[i]);
    }
  }

  return result;
}
function Trim(string){
  var result;
  var i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
  var firstNonWhitespaceFound;
  lastWhitespaceLocationStart =  -1;
  firstNonWhitespaceFound = false;
  for(i = 0; i < string.length &&  !firstNonWhitespaceFound ; i = i + 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationStart = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  /* Find whitepaces at the end. */
  lastWhitespaceLocationEnd = string.length;
  firstNonWhitespaceFound = false;
  for(i = string.length - 1; i >= 0 &&  !firstNonWhitespaceFound ; i = i - 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationEnd = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
    result = Substring(string, lastWhitespaceLocationStart + 1, lastWhitespaceLocationEnd);
  }else{
    result = [];
    result.length = 0;
  }

  return result;
}
function StartsWith(string, start){
  var startsWithString;
  startsWithString = false;
  if(string.length >= start.length){
    startsWithString = SubstringEquals(string, 0, start);
  }

  return startsWithString;
}
function EndsWith(string, end){
  var endsWithString;
  endsWithString = false;
  if(string.length >= end.length){
    endsWithString = SubstringEquals(string, string.length - end.length, end);
  }

  return endsWithString;
}
function SplitByString(toSplit, splitBy){
  var split;
  var next;
  var i;
  var c;
  var n;
  split = [];
  split.length = 0;

  next = [];
  next.length = 0;
  for(i = 0; i < toSplit.length; ){
    c = toSplit[i];
    if(SubstringEquals(toSplit, i, splitBy)){
      if(split.length != 0 || i != 0){
        n = {};
        n.string = next;
        split = AddString(split, n);
        next = [];
        next.length = 0;
        i = i + splitBy.length;
      }
    }else{
      next = AppendCharacter(next, c);
      i = i + 1;
    }
  }

  if(next.length > 0){
    n = {};
    n.string = next;
    split = AddString(split, n);
  }

  return split;
}
function AddNumber(list, a){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddNumberRef(list, i){
  list.numberArray = AddNumber(list.numberArray, i);
}
function RemoveNumber(list, n){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length - 1;

  for(i = 0; i < list.length; i = i + 1){
    if(i < n){
      newlist[i] = list[i];
    }
    if(i > n){
      newlist[i - 1] = list[i];
    }
  }
		
  delete(list);
		
  return newlist;
}
function GetNumberRef(list, i){
  return list.numberArray[i];
}
function RemoveNumberRef(list, i){
  list.numberArray = RemoveNumber(list.numberArray, i);
}
function AddString(list, a){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length + 1;

  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddStringRef(list, i){
  list.stringArray = AddString(list.stringArray, i);
}
function RemoveString(list, n){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length - 1;

  for(i = 0; i < list.length; i = i + 1){
    if(i < n){
      newlist[i] = list[i];
    }
    if(i > n){
      newlist[i - 1] = list[i];
    }
  }
		
  delete(list);
		
  return newlist;
}
function GetStringRef(list, i){
  return list.stringArray[i];
}
function RemoveStringRef(list, i){
  list.stringArray = RemoveString(list.stringArray, i);
}
function AddBoolean(list, a){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddBooleanRef(list, i){
  list.booleanArray = AddBoolean(list.booleanArray, i);
}
function RemoveBoolean(list, n){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length - 1;

  for(i = 0; i < list.length; i = i + 1){
    if(i < n){
      newlist[i] = list[i];
    }
    if(i > n){
      newlist[i - 1] = list[i];
    }
  }
		
  delete(list);
		
  return newlist;
}
function GetBooleanRef(list, i){
  return list.booleanArray[i];
}
function RemoveDecimalRef(list, i){
  list.booleanArray = RemoveBoolean(list.booleanArray, i);
}
function AddCharacter(list, a){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddCharacterRef(list, i){
  list.string = AddCharacter(list.string, i);
}
function RemoveCharacter(list, n){
  var newlist;
  var i;
  newlist = [];
  newlist.length = list.length - 1;

  for(i = 0; i < list.length; i = i + 1){
    if(i < n){
      newlist[i] = list[i];
    }
    if(i > n){
      newlist[i - 1] = list[i];
    }
  }

  delete(list);

  return newlist;
}
function GetCharacterRef(list, i){
  return list.string[i];
}
function RemoveCharacterRef(list, i){
  list.string = RemoveCharacter(list.string, i);
}
function charToLowerCase(character){
  var toReturn;
  toReturn = character;
  if(character == 'A'){
    toReturn = 'a';
  }else if(character == 'B'){
    toReturn = 'b';
  }else if(character == 'C'){
    toReturn = 'c';
  }else if(character == 'D'){
    toReturn = 'd';
  }else if(character == 'E'){
    toReturn = 'e';
  }else if(character == 'F'){
    toReturn = 'f';
  }else if(character == 'G'){
    toReturn = 'g';
  }else if(character == 'H'){
    toReturn = 'h';
  }else if(character == 'I'){
    toReturn = 'i';
  }else if(character == 'J'){
    toReturn = 'j';
  }else if(character == 'K'){
    toReturn = 'k';
  }else if(character == 'L'){
    toReturn = 'l';
  }else if(character == 'M'){
    toReturn = 'm';
  }else if(character == 'N'){
    toReturn = 'n';
  }else if(character == 'O'){
    toReturn = 'o';
  }else if(character == 'P'){
    toReturn = 'p';
  }else if(character == 'Q'){
    toReturn = 'q';
  }else if(character == 'R'){
    toReturn = 'r';
  }else if(character == 'S'){
    toReturn = 's';
  }else if(character == 'T'){
    toReturn = 't';
  }else if(character == 'U'){
    toReturn = 'u';
  }else if(character == 'V'){
    toReturn = 'v';
  }else if(character == 'W'){
    toReturn = 'w';
  }else if(character == 'X'){
    toReturn = 'x';
  }else if(character == 'Y'){
    toReturn = 'y';
  }else if(character == 'Z'){
    toReturn = 'z';
  }

  return toReturn;
}
function charToUpperCase(character){
  var toReturn;
  toReturn = character;
  if(character == 'a'){
    toReturn = 'A';
  }else if(character == 'b'){
    toReturn = 'B';
  }else if(character == 'c'){
    toReturn = 'C';
  }else if(character == 'd'){
    toReturn = 'D';
  }else if(character == 'e'){
    toReturn = 'E';
  }else if(character == 'f'){
    toReturn = 'F';
  }else if(character == 'g'){
    toReturn = 'G';
  }else if(character == 'h'){
    toReturn = 'H';
  }else if(character == 'i'){
    toReturn = 'I';
  }else if(character == 'j'){
    toReturn = 'J';
  }else if(character == 'k'){
    toReturn = 'K';
  }else if(character == 'l'){
    toReturn = 'L';
  }else if(character == 'm'){
    toReturn = 'M';
  }else if(character == 'n'){
    toReturn = 'N';
  }else if(character == 'o'){
    toReturn = 'O';
  }else if(character == 'p'){
    toReturn = 'P';
  }else if(character == 'q'){
    toReturn = 'Q';
  }else if(character == 'r'){
    toReturn = 'R';
  }else if(character == 's'){
    toReturn = 'S';
  }else if(character == 't'){
    toReturn = 'T';
  }else if(character == 'u'){
    toReturn = 'U';
  }else if(character == 'v'){
    toReturn = 'V';
  }else if(character == 'w'){
    toReturn = 'W';
  }else if(character == 'x'){
    toReturn = 'X';
  }else if(character == 'y'){
    toReturn = 'Y';
  }else if(character == 'z'){
    toReturn = 'Z';
  }

  return toReturn;
}
function charIsUpperCase(character){
  var isUpper;
  isUpper = false;
  if(character == 'A'){
    isUpper = true;
  }else if(character == 'B'){
    isUpper = true;
  }else if(character == 'C'){
    isUpper = true;
  }else if(character == 'D'){
    isUpper = true;
  }else if(character == 'E'){
    isUpper = true;
  }else if(character == 'F'){
    isUpper = true;
  }else if(character == 'G'){
    isUpper = true;
  }else if(character == 'H'){
    isUpper = true;
  }else if(character == 'I'){
    isUpper = true;
  }else if(character == 'J'){
    isUpper = true;
  }else if(character == 'K'){
    isUpper = true;
  }else if(character == 'L'){
    isUpper = true;
  }else if(character == 'M'){
    isUpper = true;
  }else if(character == 'N'){
    isUpper = true;
  }else if(character == 'O'){
    isUpper = true;
  }else if(character == 'P'){
    isUpper = true;
  }else if(character == 'Q'){
    isUpper = true;
  }else if(character == 'R'){
    isUpper = true;
  }else if(character == 'S'){
    isUpper = true;
  }else if(character == 'T'){
    isUpper = true;
  }else if(character == 'U'){
    isUpper = true;
  }else if(character == 'V'){
    isUpper = true;
  }else if(character == 'W'){
    isUpper = true;
  }else if(character == 'X'){
    isUpper = true;
  }else if(character == 'Y'){
    isUpper = true;
  }else if(character == 'Z'){
    isUpper = true;
  }

  return isUpper;
}
function charIsLowerCase(character){
  var isLower;
  isLower = false;
  if(character == 'a'){
    isLower = true;
  }else if(character == 'b'){
    isLower = true;
  }else if(character == 'c'){
    isLower = true;
  }else if(character == 'd'){
    isLower = true;
  }else if(character == 'e'){
    isLower = true;
  }else if(character == 'f'){
    isLower = true;
  }else if(character == 'g'){
    isLower = true;
  }else if(character == 'h'){
    isLower = true;
  }else if(character == 'i'){
    isLower = true;
  }else if(character == 'j'){
    isLower = true;
  }else if(character == 'k'){
    isLower = true;
  }else if(character == 'l'){
    isLower = true;
  }else if(character == 'm'){
    isLower = true;
  }else if(character == 'n'){
    isLower = true;
  }else if(character == 'o'){
    isLower = true;
  }else if(character == 'p'){
    isLower = true;
  }else if(character == 'q'){
    isLower = true;
  }else if(character == 'r'){
    isLower = true;
  }else if(character == 's'){
    isLower = true;
  }else if(character == 't'){
    isLower = true;
  }else if(character == 'u'){
    isLower = true;
  }else if(character == 'v'){
    isLower = true;
  }else if(character == 'w'){
    isLower = true;
  }else if(character == 'x'){
    isLower = true;
  }else if(character == 'y'){
    isLower = true;
  }else if(character == 'z'){
    isLower = true;
  }

  return isLower;
}
function charIsLetter(character){
  return charIsUpperCase(character) || charIsLowerCase(character);
}
function charIsNumber(character){
  var isNumber;
  isNumber = false;
  if(character == '0'){
    isNumber = true;
  }else if(character == '1'){
    isNumber = true;
  }else if(character == '2'){
    isNumber = true;
  }else if(character == '3'){
    isNumber = true;
  }else if(character == '4'){
    isNumber = true;
  }else if(character == '5'){
    isNumber = true;
  }else if(character == '6'){
    isNumber = true;
  }else if(character == '7'){
    isNumber = true;
  }else if(character == '8'){
    isNumber = true;
  }else if(character == '9'){
    isNumber = true;
  }

  return isNumber;
}
function charIsWhiteSpace(character){
  var isWhiteSpace;
  isWhiteSpace = false;
  if(character == ' '){
    isWhiteSpace = true;
  }else if(character == '\t'){
    isWhiteSpace = true;
  }else if(character == '\n'){
    isWhiteSpace = true;
  }else if(character == '\r'){
    isWhiteSpace = true;
  }

  return isWhiteSpace;
}
function charIsSymbol(character){
  var isSymbol;
  isSymbol = false;
  if(character == '!'){
    isSymbol = true;
  }else if(character == '\"'){
    isSymbol = true;
  }else if(character == '#'){
    isSymbol = true;
  }else if(character == '$'){
    isSymbol = true;
  }else if(character == '%'){
    isSymbol = true;
  }else if(character == '&'){
    isSymbol = true;
  }else if(character == '\''){
    isSymbol = true;
  }else if(character == '('){
    isSymbol = true;
  }else if(character == ')'){
    isSymbol = true;
  }else if(character == '*'){
    isSymbol = true;
  }else if(character == '+'){
    isSymbol = true;
  }else if(character == ','){
    isSymbol = true;
  }else if(character == '-'){
    isSymbol = true;
  }else if(character == '.'){
    isSymbol = true;
  }else if(character == '/'){
    isSymbol = true;
  }else if(character == ':'){
    isSymbol = true;
  }else if(character == ';'){
    isSymbol = true;
  }else if(character == '<'){
    isSymbol = true;
  }else if(character == '='){
    isSymbol = true;
  }else if(character == '>'){
    isSymbol = true;
  }else if(character == '?'){
    isSymbol = true;
  }else if(character == '@'){
    isSymbol = true;
  }else if(character == '['){
    isSymbol = true;
  }else if(character == '\\'){
    isSymbol = true;
  }else if(character == ']'){
    isSymbol = true;
  }else if(character == '^'){
    isSymbol = true;
  }else if(character == '_'){
    isSymbol = true;
  }else if(character == '`'){
    isSymbol = true;
  }else if(character == '{'){
    isSymbol = true;
  }else if(character == '|'){
    isSymbol = true;
  }else if(character == '}'){
    isSymbol = true;
  }else if(character == '~'){
    isSymbol = true;
  }

  return isSymbol;
}


module.exports = {
	OrdinaryLeastSquares: OrdinaryLeastSquares,
	OrdinaryLeastSquaresWithIntercept: OrdinaryLeastSquaresWithIntercept,
	Add: Add,
	Assign: Assign,
	Subtract: Subtract,
	SubtractToNew: SubtractToNew,
	ScalarMultiply: ScalarMultiply,
	ScalarDivide: ScalarDivide,
	ElementWisePower: ElementWisePower,
	ScalarMultiplyToNew: ScalarMultiplyToNew,
	MultiplyToNew: MultiplyToNew,
	Multiply: Multiply,
	CreateSquareMatrix: CreateSquareMatrix,
	CreateMatrix: CreateMatrix,
	CreateIdentityMatrix: CreateIdentityMatrix,
	Transpose: Transpose,
	TransposeToNew: TransposeToNew,
	CofactorOfMatrix: CofactorOfMatrix,
	DeterminantOfSubmatrix: DeterminantOfSubmatrix,
	Determinant: Determinant,
	Adjoint: Adjoint,
	Inverse: Inverse,
	IsSymmetric: IsSymmetric,
	IsSquare: IsSquare,
	Cholesky: Cholesky,
	Clear: Clear,
	Fill: Fill,
	Element: Element,
	Trace: Trace,
	ColumnCombineMatricesToNew: ColumnCombineMatricesToNew,
	NumberOfRows: NumberOfRows,
	NumberOfColumns: NumberOfColumns,
	CharacteristicPolynomial: CharacteristicPolynomial,
	CharacteristicPolynomialWithInverse: CharacteristicPolynomialWithInverse,
	InverseAlt2: InverseAlt2,
	Eigenvalues: Eigenvalues,
	GaussianElimination: GaussianElimination,
	GaussianEliminationToNew: GaussianEliminationToNew,
	CreateCopyOfMatrix: CreateCopyOfMatrix,
	SwapRows: SwapRows,
	Eigenvectors: Eigenvectors,
	RowIsZero: RowIsZero,
	DeleteMatrix: DeleteMatrix,
	CreateDiagonalMatrixFromArray: CreateDiagonalMatrixFromArray,
	CreateMatrixFromRowCopies: CreateMatrixFromRowCopies,
	ExtractDiagonal: ExtractDiagonal,
	MatrixEqualsEpsilon: MatrixEqualsEpsilon,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	FindRoots: FindRoots,
	LaguerresMethodWithRepeatedDivision: LaguerresMethodWithRepeatedDivision,
	LaguerresMethod: LaguerresMethod,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	EpsilonCompareApproximateDigits: EpsilonCompareApproximateDigits,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	StringToNumberArray: StringToNumberArray,
	NumberArrayToString: NumberArrayToString,
	NumberArraysEqual: NumberArraysEqual,
	BooleanArraysEqual: BooleanArraysEqual,
	StringsEqual: StringsEqual,
	FillNumberArray: FillNumberArray,
	FillString: FillString,
	FillBooleanArray: FillBooleanArray,
	FillNumberArrayInterval: FillNumberArrayInterval,
	FillBooleanArrayInterval: FillBooleanArrayInterval,
	FillStringInterval: FillStringInterval,
	CopyNumberArray: CopyNumberArray,
	CopyBooleanArray: CopyBooleanArray,
	CopyString: CopyString,
	CopyNumberArrayRange: CopyNumberArrayRange,
	CopyBooleanArrayRange: CopyBooleanArrayRange,
	CopyStringRange: CopyStringRange,
	IsLastElement: IsLastElement,
	CreateNumberArray: CreateNumberArray,
	CreateBooleanArray: CreateBooleanArray,
	CreateString: CreateString,
	pAdd: pAdd,
	pSubtract: pSubtract,
	pMultiply: pMultiply,
	pDivide: pDivide,
	pIsZero: pIsZero,
	pAssign: pAssign,
	pCreatePolynomial: pCreatePolynomial,
	pFill: pFill,
	pDegree: pDegree,
	pLead: pLead,
	pEvaluate: pEvaluate,
	pEvaluateDerivative: pEvaluateDerivative,
	nCreateStringDecimalFromNumber: nCreateStringDecimalFromNumber,
	nCreateStringFromNumberWithCheck: nCreateStringFromNumberWithCheck,
	nGetMaximumDigitsForBase: nGetMaximumDigitsForBase,
	nGetFirstDigitPosition: nGetFirstDigitPosition,
	nGetSingleDigitCharacterFromNumberWithCheck: nGetSingleDigitCharacterFromNumberWithCheck,
	nGetDigitCharacterTable: nGetDigitCharacterTable,
	nCreateNumberFromDecimalStringWithCheck: nCreateNumberFromDecimalStringWithCheck,
	nCreateNumberFromDecimalString: nCreateNumberFromDecimalString,
	nCreateNumberFromStringWithCheck: nCreateNumberFromStringWithCheck,
	nGetNumberFromNumberCharacterForBase: nGetNumberFromNumberCharacterForBase,
	nCharacterIsNumberCharacterInBase: nCharacterIsNumberCharacterInBase,
	nStringToNumberArray: nStringToNumberArray,
	nStringToNumberArrayWithCheck: nStringToNumberArrayWithCheck,
	SubstringWithCheck: SubstringWithCheck,
	Substring: Substring,
	AppendString: AppendString,
	ConcatenateString: ConcatenateString,
	AppendCharacter: AppendCharacter,
	ConcatenateCharacter: ConcatenateCharacter,
	SplitByCharacter: SplitByCharacter,
	IndexOfCharacter: IndexOfCharacter,
	SubstringEqualsWithCheck: SubstringEqualsWithCheck,
	SubstringEquals: SubstringEquals,
	IndexOfString: IndexOfString,
	ContainsCharacter: ContainsCharacter,
	ContainsString: ContainsString,
	ToUpperCase: ToUpperCase,
	ToLowerCase: ToLowerCase,
	EqualsIgnoreCase: EqualsIgnoreCase,
	ReplacesString: ReplacesString,
	ReplaceCharacter: ReplaceCharacter,
	Trim: Trim,
	StartsWith: StartsWith,
	EndsWith: EndsWith,
	SplitByString: SplitByString,
	AddNumber: AddNumber,
	AddNumberRef: AddNumberRef,
	RemoveNumber: RemoveNumber,
	GetNumberRef: GetNumberRef,
	RemoveNumberRef: RemoveNumberRef,
	AddString: AddString,
	AddStringRef: AddStringRef,
	RemoveString: RemoveString,
	GetStringRef: GetStringRef,
	RemoveStringRef: RemoveStringRef,
	AddBoolean: AddBoolean,
	AddBooleanRef: AddBooleanRef,
	RemoveBoolean: RemoveBoolean,
	GetBooleanRef: GetBooleanRef,
	RemoveDecimalRef: RemoveDecimalRef,
	AddCharacter: AddCharacter,
	AddCharacterRef: AddCharacterRef,
	RemoveCharacter: RemoveCharacter,
	GetCharacterRef: GetCharacterRef,
	RemoveCharacterRef: RemoveCharacterRef,
	charToLowerCase: charToLowerCase,
	charToUpperCase: charToUpperCase,
	charIsUpperCase: charIsUpperCase,
	charIsLowerCase: charIsLowerCase,
	charIsLetter: charIsLetter,
	charIsNumber: charIsNumber,
	charIsWhiteSpace: charIsWhiteSpace,
	charIsSymbol: charIsSymbol,
	test: test
};